### Packages: Library installation
#source("http://www.bioconductor.org/biocLite.R")
#biocLite("DESeq2")
#biocLite("edgeR")
#install.packages("FactoMineR")
#install.packages("VennDiagram")
#install.packages('dendextend')

### Library calling
library(DESeq2)
library(edgeR)
library(FactoMineR)
library(VennDiagram)
library(dendextend)

RawCounts <- read.delim("TableS2.csv", header=TRUE, fileEncoding="UTF-8-BOM", sep=";")
PlanDexp <- read.delim("TableS1.csv", header=TRUE, fileEncoding="UTF-8-BOM", sep=";")

###Dataset formatting
RawCounts$Gene<-as.character(RawCounts$Gene)
RawCounts$Transcript<-as.character(RawCounts$Transcript)

Counts <- RawCounts[,4:28]
Counts <- as.matrix(Counts)
rownames(Counts)<-RawCounts[,2]
Counts <- Counts[,as.character(PlanDexp$Name)]
Geno <- as.factor(paste(PlanDexp$Line, PlanDexp$Genotype,sep=""))
An <- as.factor (PlanDexp$Year)
Stade <- as.factor((PlanDexp$Status))
GxS <- paste(Geno,Stade,sep="_")
GxA <- paste(Geno,An,sep="_")
SxA <- paste(Stade,An,sep="_")
fact <- paste(Geno, An, Stade, GxS, GxA, SxA,sep=":")

###Data filtering and normalization by size factors
### Filtering genes with low expression
geneswithcounts <- rowSums (Counts) >= 1  
keep <- rowSums (cpm(Counts) > 1) >= (dim(Counts)[2]/2) 
Counts <- Counts[keep,] 
Cond <- data.frame(An, GxS)
rownames(Cond)<-colnames(Counts)
Tab <- DESeqDataSetFromMatrix(Counts, colData=Cond, formula(~An+GxS))
sf <- estimateSizeFactors(Tab)
sfList <- sizeFactors(sf)
attributes(sf)

###Table of normalized counts
Countsf<-Counts[,1]/sf$sizeFactor[1]
for(colonne in 2:dim(Counts)[2]){
  Countsf<-cbind(Countsf,Counts[,colonne]/sf$sizeFactor[colonne])
  }
colnames(Countsf)<-colnames(Counts)

### Correlations among libraries
pdf("correl_rep_log.pdf")
for(g in 1:nlevels(Geno)){
  sel<-Geno==levels(Geno)[g]
  pairs(log(Countsf[,sel]), pch=".", cex=3, upper.panel = NULL,  main = "Correlation normalisation DESeq",oma=c(0,0,0,0))
}
dev.off()
tabcorel <- cor(Countsf)
range(tabcorel[tabcorel!=1])

### Comparison of quantiles before and after normalization
plot_quantiles <- function(CountTable,mytitle){
  Q <- apply(CountTable,2,quantile)
  med <- Q[3,]
  q25 <- Q[2,]
  q75 <- Q[4,]
  ylimits <- range(c(med,q25,q75))
  plot(1:dim(Q)[2],med,type="p",pch=19,xaxt="n",xlab="",ylab="Counts",ylim=ylimits)
  for (i in 1:dim(Q)[2]){
    lines(c(i,i),c(q25[i],q75[i]))
    mtext(side=1,at=i,line=1,colnames(Q)[i],las=3)
  }
  title(mytitle)
}
pdf("normalization_quantiles.pdf")
par(mfrow=c(1,1),mar=c(8,2,2,2))
plot_quantiles(Countsf,"DESeq2 normalization")
dev.off()

 
### Design matrix
design_matrix <- model.matrix(~An+GxS)
temp <- colnames(design_matrix)
temp <- gsub("GxS","",temp)
temp <- gsub("arly","",temp)
temp <- gsub("ate","",temp)
colnames(design_matrix)<-temp

#Year contrast
cont.mat_an <- makeContrasts(An2, levels=design_matrix)

#Status contrasts
cont.mat_F_VT <- makeContrasts((FE_V+FL_V+FVL_V)-(FE_T+FL_T+FVL_T), levels=design_matrix)
cont.mat_F_VR <- makeContrasts((FE_V+FL_V+FVL_V)-(FL_R+FVL_R), levels=design_matrix)
cont.mat_F_TR <- makeContrasts((FL_T+FVL_T)-(FL_R+FVL_R), levels=design_matrix)

cont.mat_M_VT <- makeContrasts((ME_V)-(ME_T+ML_T), levels=design_matrix)
cont.mat_M_VR <- makeContrasts((ME_V)-(ME_R+ML_R), levels=design_matrix)
cont.mat_M_TR <- makeContrasts((ME_T+ML_T)-(ME_R+ML_R), levels=design_matrix)

#Progenitor contrasts
cont.mat_F_EL <- makeContrasts((FE_V+FE_T)-(FL_V+FL_T+FL_R), levels=design_matrix)
cont.mat_F_EVL <- makeContrasts((FE_V+FE_T)-(FVL_V+FVL_T+FVL_R), levels=design_matrix)
cont.mat_M_EL <- makeContrasts((ME_V+ME_T+ME_R)-(ML_T+ML_R), levels=design_matrix)

#Line contrasts
cont.mat_FM_E <- makeContrasts((FE_V+FE_T)-(ME_V+ME_T+ME_R), levels=design_matrix)
cont.mat_FM_L <- makeContrasts((FL_V+FL_T+FL_R)-(ML_T+ML_R), levels=design_matrix)
cont.mat_FM <- makeContrasts((FE_V+FE_T+FL_V+FL_T+FL_R+FVL_V+FVL_T+FVL_R)-(ME_V+ME_T+ME_R+ML_R+ML_T), levels=design_matrix)
cont.mat_FVLM_L <- makeContrasts((FVL_V+FVL_T+FVL_R)-(ML_T+ML_R), levels=design_matrix)

#Status x Progenitor contrasts:
cont.mat_FE_VT <- makeContrasts (FE_V-FE_T, levels=design_matrix)
cont.mat_FE_VR <- makeContrasts (FE_V, levels=design_matrix)
cont.mat_FE_TR <- makeContrasts (FE_T, levels=design_matrix)

cont.mat_FL_VT <- makeContrasts (FL_V-FL_T, levels=design_matrix)
cont.mat_FL_VR <- makeContrasts (FL_V, levels=design_matrix)
cont.mat_FL_TR <- makeContrasts (FL_T, levels=design_matrix)

cont.mat_FVL_VT <- makeContrasts (FVL_V-FVL_T, levels=design_matrix)
cont.mat_FVL_VR <- makeContrasts (FVL_V-FVL_R, levels=design_matrix)
cont.mat_FVL_TR <- makeContrasts (FVL_T-FVL_R, levels=design_matrix)

cont.mat_ME_VT <- makeContrasts (ME_V-ME_T, levels=design_matrix)
cont.mat_ME_VR <- makeContrasts (ME_V-ME_R, levels=design_matrix)
cont.mat_ME_TR <- makeContrasts (ME_T-ME_R, levels=design_matrix)

cont.mat_ML_TR <- makeContrasts (ML_T-ML_R, levels=design_matrix)


cont.mat<-rbind(c(cont.mat_an),c(cont.mat_F_VT),c(cont.mat_F_VR),c(cont.mat_F_TR),
                c(cont.mat_M_VT),c(cont.mat_M_VR),c(cont.mat_M_TR),
                c(cont.mat_F_EL),c(cont.mat_F_EVL),c(cont.mat_M_EL),c(cont.mat_FM_E),
                c(cont.mat_FM_L),c(cont.mat_FM),c(cont.mat_FVLM_L),
                c(cont.mat_FE_VT),c(cont.mat_FE_VR),c(cont.mat_FE_TR),
                c(cont.mat_FL_VT),c(cont.mat_FL_VR),c(cont.mat_FL_TR),c(cont.mat_FVL_VT),
                c(cont.mat_FVL_VR),c(cont.mat_FVL_TR),c(cont.mat_ME_VT),c(cont.mat_ME_VR),
                c(cont.mat_ME_TR),c(cont.mat_ML_TR))
rownames(cont.mat)<-c("an","F_VT","F_VR","F_TR","M_VT","M_VR","M_TR","F_EL","F_EVL","M_EL","FM_E",
                      "FM_L","FM","FVLM_L","FE_VT","FE_VR","FE_TR","FL_VT","FL_VR","FL_TR","FVL_VT",
                      "FVL_VR","FVL_TR","ME_VT","ME_VR","ME_TR","ML_TR")

### DE analysis
design(Tab) <- ~An+GxS
mydeseq <- DESeq(Tab,betaPrior=FALSE)
resultsNames(mydeseq)

### P-value distribution and storage of test results
tabR <- NULL
tabP <- NULL
pdf("pval_distrib.pdf")
p_value<-NULL
nbGenetot<-NULL
for (i in 1:dim(cont.mat)[1]){
  test <- results(mydeseq, contrast=cont.mat[i,])
  nbGenetot<-c(nbGenetot,dim(test)[1])
  test <- as.data.frame(test)
  test_padj <- test[test$padj < 0.05,]
  test_padj <- na.omit(test_padj)
  GDE <- rownames(test_padj)
  print(rownames(cont.mat)[i])
  p_value<-c(p_value,test$pvalue)
  hist(test$pvalue, 50, xlab="", main=rownames(cont.mat)[i], ylim=c(0,4*10^3))
  tabP <- c(tabP,test$pvalue)
  if (length(GDE)>0){
    tabR <- rbind.data.frame(tabR,cbind.data.frame(gene=GDE,contrast=rownames(cont.mat)[i],test[GDE,]))
  }
}
dev.off()
dim(tabR)

### Data formating
ListGenes<-as.character(tabR$gene)
temp<-matrix(unlist(strsplit(ListGenes,"\\_")),ncol=2,byrow=TRUE)
colnames(temp)<-c("ge","tr")
temp<-data.frame(temp)
tabR$GeneNames<-temp$ge

### Analysis of FT_candidates
Cand<-read.table("TableS5_FT_Candidates.csv", fileEncoding="UTF-8-BOM", sep=";", header=TRUE, strip.white = FALSE)
Cand<-Cand[,1:2]
ContrastCand<-tabR[which(tabR$GeneNames%in%as.character(Cand$Gene)),]
cand.list <- as.character(Cand$Gene)
length(unique(ContrastCand$gene))
cand.DE <- as.character(unique(ContrastCand$gene))
CandCounts <- RawCounts[Cand$Gene,]
CandCounts[!rownames(CandCounts)%in%unique(ContrastCand$gene),]

###Comparison of level of expression of all genes versus FT_candidates
Candsum<-apply(CandCounts[,-c(1:3)],1,sum)
Rawsum<-apply(RawCounts[,-c(1:3)],1,sum)
par(mfrow=c(2,1))
rawprops <- hist(log(Rawsum),plot=F,breaks=seq(0,14,length=10))$counts
candprops <- hist(log(Candsum),plot=F,breaks=seq(0,14,length=10))$counts
chisq.test(candprops,p=rawprops/sum(rawprops),simulate.p.value=T,B=5000)

###Analysis of GWA_candidates
GWA_Cand<-read.table("TableS5_GWA_candidates.csv", fileEncoding="UTF-8-BOM", sep=";", colClasses="character", header=TRUE, strip.white = FALSE)
GWA_Cand<-GWA_Cand[,1:2]
ContrastCand<-tabR[which(as.character(tabR$GeneNames)%in%as.character(GWA_Cand$Gene)),]
length(unique(ContrastCand$gene))
GWA.DE<-as.character(unique(ContrastCand$gene))

###Finding the number of DE genes by contrast
GenesPerContrast<-matrix(NA,ncol=3,nrow=dim(cont.mat)[1])
colnames(GenesPerContrast) <- c("Genes","DE","Cand")
GenesPerContrast <- data.frame(GenesPerContrast)
levels(tabR$contrast)
levels (ContrastCand$contrast)
for (i in 1:dim(cont.mat)[1]){
  GenesPerContrast$Genes[i]<-rownames(cont.mat)[i]
  GenesPerContrast$DE[i]<-length(tabR$contrast[tabR$contrast==rownames(cont.mat)[i]])
  GenesPerContrast$Cand[i]<-length(ContrastCand$contrast[ContrastCand$contrast==rownames(cont.mat)[i]])
  }
print(nbGenetot)
GenesPerContrast[GenesPerContrast[,"DE"]==0,]
write.table(GenesPerContrast, file = "GenesPerContrast.csv", sep = "\t", na = "NA", dec = ".", row.names = TRUE, col.names = TRUE)

###Data centering and correction for Year effect
DEgenes <- unique(as.character(tabR$gene), incomparables = FALSE)
length(DEgenes)
matrixGSmeans<-NULL
vectSD <- NULL
for (i in 1:length(DEgenes)){
  y<-Countsf[rownames(Countsf)==DEgenes[i],]
  mylm <- lm(y~0+GxS+An)
  an0 <- mylm$coef[-length(mylm$coef)]
  an1 <- an0 + mylm$coef[length(mylm$coef)]
  GSmeans<-(an0+an1)/2
  matrixGSmeans<-rbind(matrixGSmeans,GSmeans)
  vectSD <- c(vectSD,summary(mylm)$sigma)
  }
rownames(matrixGSmeans) <- DEgenes
names(vectSD) <- DEgenes
preds<-matrixGSmeans
m <- apply(preds,1,mean)
tabm <- matrix(rep(m,each=ncol(preds)),ncol=ncol(preds),byrow=TRUE)
Zsignif <- preds-tabm


### Data saving
save(Countsf, cont.mat, tabR, Zsignif, preds, vectSD,file="DEanalysis.Rdata")
library(DESeq2)
library(edgeR)
load("DEanalysis.Rdata")


###Table of significant contrasts
contrasts.names <- levels(tabR$contrast)
CTmat <- matrix(0,nrow=nrow(Zsignif),ncol=length(contrasts.names))
rownames(CTmat) <- rownames(Zsignif)
colnames(CTmat) <- contrasts.names
  for (g in rownames(Zsignif)){
    thecontrasts <- as.character(tabR[tabR$gene==g,"contrast"])
    CTmat[g,thecontrasts] <- 1
    }

###Number of significant contrasts per gene
nbCT <- apply(CTmat,1,sum)
table(nbCT)
  
###Contrast categories
contrasts.categ <- c("year","FTF","FTF","FTF","MTF","MTF","FEL","FEL","MEL","FM","FM","FM","FM","GS","GS","GS","GS","GS","GS","GS","GS","GS","GS","GS","GS")
#Year category
Cy <- CTmat[,contrasts.categ=="year"]
#Status category
Cftf <- apply(CTmat[,contrasts.categ=="FTF"],1,sum)>0
Cmtf <- apply(CTmat[,contrasts.categ=="MTF"],1,sum)>0
Ctf <- apply(CTmat[,contrasts.categ%in%c("FTF","MTF")],1,sum)>0
#Selection category
Cfel <- apply(CTmat[,contrasts.categ=="FEL"],1,sum)>0
Cmel <- CTmat[,contrasts.categ=="MEL"]>0
#Line category
Cfm <- apply(CTmat[,contrasts.categ=="FM"],1,sum)>0
# Progenitor x Status category
Cgs <- apply(CTmat[,contrasts.categ=="GS"],1,sum)>0
# Selection within Progenitor x Status category
Cgs.fel <- CTmat[,"FE_VT"]==1 & CTmat[,"FL_VT"]==0 |
      CTmat[,"FE_VR"]==1 & CTmat[,"FL_VR"]==0 |
      CTmat[,"FE_TR"]==1 & CTmat[,"FL_TR"]==0 |
      CTmat[,"FE_VT"]==0 & CTmat[,"FL_VT"]==1 |
      CTmat[,"FE_VR"]==0 & CTmat[,"FL_VR"]==1 |
      CTmat[,"FE_TR"]==0 & CTmat[,"FL_TR"]==1
  
Cgs.fevl <- CTmat[,"FE_VT"]==1 & CTmat[,"FVL_VT"]==0 |
    CTmat[,"FE_VT"]==1 & CTmat[,"FVL_VT"]==0 |
    CTmat[,"FE_VR"]==1 & CTmat[,"FVL_VR"]==0 |
    CTmat[,"FE_TR"]==1 & CTmat[,"FVL_TR"]==0 |
    CTmat[,"FE_VT"]==0 & CTmat[,"FVL_VT"]==1 |
    CTmat[,"FE_VR"]==0 & CTmat[,"FVL_VR"]==1 |
    CTmat[,"FE_TR"]==0 & CTmat[,"FVL_TR"]==1
Cgs.f <- Cgs.fel|Cgs.fevl

###Table with contrast categories
CmN <- cbind.data.frame(Y=Cy*1,Lig=Cfm*1, Ftf=Cftf*1,Mtf=Cmtf*1,FEL=Cfel*1,MEL=Cmel*1,gsO = Cgs*1,gsF=Cgs.f*1)
CmN.typ <- apply(CmN,1,paste,collapse=":")
N <- table(CmN.typ)
tempN <- matrix(unlist(strsplit(names(N),":")),ncol=8,byrow=TRUE)
tabHN <- cbind(tempN,N)
write.table(tabHN,file="types_contrasts.csv",sep=";")
  
### Suppression of DE genes with Year effect only, Line effect only, or Year+Line only
yearonly <- apply(CTmat,1,sum)==1 & CTmat[,"an"]==1
ficyear<-cbind(CTmat[,1],yearonly)
ficyear<-as.data.frame(ficyear)
write.table(ficyear,file="yearonly.csv",sep=";")
  
lineonly <- apply(CTmat[,contrasts.categ=="FM"],1,sum)>0 & apply(CTmat[,contrasts.categ!="FM"],1,sum)==0
ficline<-cbind(CTmat[,contrasts.categ=="FM"],lineonly)
ficline<-as.data.frame(ficline)
write.table(ficline,file="lineonly.csv",sep=";")
  
LinePlusYear<- apply(CTmat[,contrasts.categ=="FM"],1,sum)>0 & 
apply(CTmat[,contrasts.categ!="FM" & contrasts.categ!="year"],1,sum)==0 &
CTmat[,contrasts.categ=="year"]==1
  
CTmat_all <- CTmat
Zsignif_all <- Zsignif

###Analysis of DE genes of the selection category, Venn diagram
  
ELgenes <- rownames(CTmat_all)[Cfel | Cmel | Cgs.fel | Cgs.fevl]
CC <- cbind(FEL=CTmat_all[,"F_EL"],gsFEL=Cgs.fel,FEVL=CTmat_all[,"F_EVL"],gsFEVL=Cgs.fevl,MEL=CTmat_all[,"M_EL"])
write.table(CC,file="ELgenes.csv",sep=";")

v <- vennCounts(CC)
pdf("vennEL.pdf")
vennDiagram(v,names="",show.include=FALSE)
dev.off() 
  
ZEL <- Zsignif[ELgenes,] 
save(ZEL,file="ZEL.RData")
temp2<-matrix(unlist(strsplit(ELgenes,"\\_")),ncol=2,byrow=TRUE)
colnames(temp2)<-c("ge","tr")
temp2<-data.frame(temp2)
NbCandWithinEL<-(temp2$ge%in%Cand$gene)
sum(NbCandWithinEL)
write.table(ELgenes[NbCandWithinEL],file="ListeEL_withinCand.csv",sep=";")

###Functional annotation
kegg<-read.table("TableS10.csv",header=TRUE,sep=";",fileEncoding="UTF-8-BOM")
kegg.genes <- toupper(as.character(kegg$Tr.name))
de_genes <- rownames(Zsignif_all)
sum(de_genes%in%kegg.genes)
de_genes[!de_genes%in%kegg.genes]
sum(kegg.genes%in%de_genes)
tabAnnot <- kegg[kegg.genes%in%de_genes,]
genes.annot <- kegg.genes[kegg.genes%in%de_genes]
tabAnnot$Transcript <- genes.annot
tabAnnot$mapman.bin <- paste(tabAnnot$bin.id1,tabAnnot$bin.id2,sep=":")
tabAnnot <- tabAnnot[,c("Transcript","bin.main","mapman.bin","bin.name","kegg1","kegg2","kegg3")]
  
###PCA
Zsignif <- Zsignif[!lineonly & !yearonly & !LinePlusYear,]
tZsignif <- t(Zsignif)
acp <- prcomp(tZsignif,scale.=TRUE)
var.exp <- acp$sdev^2/sum(acp$sdev^2)
conds <- rownames(tZsignif)
nbaxes <- min(length(conds),dim(tZsignif)[2])-1
pdf("acp.pdf")
par(mfrow=c(1,1),xpd=TRUE,mar=c(4,4,2,2))
mypch <- c(19,17)
names(mypch) <- c("F","M")
pchACP<-c("F","F","F","F","F","F","F","F","F","M","M","M","M","M")
legend<-c("E","E","E","L","L","L","VL","VL","VL","E","E","E","L","L")
colorACP<-c("black","red","green","black","red","green","black","red","green","black","red","green","black","red")
i=2
plot(acp$x[,1],acp$x[,i],type="n",xlab=paste("PCA1: ",round(100*var.exp[1],0),"%",sep=""),ylab=paste("PCA",i,": ",round(100*var.exp[i],0),"%",sep=""))
points(acp$x[,1],acp$x[,i],pch=mypch[pchACP],col=colorACP)
dev.off()

###correlation with PCA axes and attribution of genes to a single axis
nbgenes <- dim(tZsignif)[2]
acpcor <- NULL
acpp <- NULL
  for (i in 1:nbgenes){
   ri <- NULL
   pvali <- NULL 
     for (j in 1:nbaxes){
      r <- cor.test(acp$x[,j],tZsignif[,i])
      ri <- c(ri,r$est)
      pvali <- c(pvali,r$p.val) 
     }
   acpcor <- rbind(acpcor,ri)
   acpp <- rbind(acpp,pvali)
  }
rownames(acpcor) <- colnames(tZsignif)
rownames(acpp) <- colnames(tZsignif)
acpcol <- apply(abs(acpcor),1,which.max)
table(acpcol)
write.table(acpcol, file = "acpcol.csv", sep = "\t", na = "NA", dec = ".", row.names = TRUE, col.names = TRUE)

###Table S4
colnames(preds) <- gsub("GxS","",colnames(preds))
colorder <- c("FEarly_V","FEarly_T","FEarly_R","FLate_V","FLate_T","FLate_R","FVLate_V","FVLate_T","FVLate_R","MEarly_V","MEarly_T","MEarly_R","MLate_T","MLate_R")
preds <- preds[,colorder]
vectmoy <- apply(preds,1,mean)
tabS4 <- cbind.data.frame(average=vectmoy,sd=vectSD,preds)
thegenes <- rownames(tabS4)
tabS4$contrast <- ""
tabS4$contrast[yearonly[thegenes]==1] <- "[Y]"
tabS4$contrast[lineonly[thegenes]==1] <- "[L]"
tabS4$contrast[LinePlusYear[thegenes]==1] <- "[Y][L]"
mela <- Cmel[thegenes]==1 & Cfel[thegenes]==0 & Cgs.f[thegenes]==0
melb <- Cmel[thegenes]==1 & !mela
tabS4$contrast[mela] <- "[SelM]"
tabS4$contrast[melb] <- "[SelF]+[SelM]"
fel <- (Cfel[thegenes]==1 | Cgs.f[thegenes]==1) & !melb
tabS4$contrast[fel] <- "[SelF]"
subgenes <- thegenes[thegenes%in%names(acpcol)]
tabS4$pcagroup <- NA
tabS4$pcagroup[thegenes%in%names(acpcol)] <- acpcol[subgenes]
tabA <- NULL
  for (i in 1:dim(tabS4)[1]){
    A <- tabAnnot[tabAnnot$Transcript==rownames(tabS4)[i],]
      if (dim(A)[1]==1){
      tabA <- rbind.data.frame(tabA,A)
      } else if (dim(A)[1]>1){
        B <- rep(NA,dim(A)[1])
        B[1] <- A[1,1]
        for (k in 2:dim(A)[2]){
          B[k] <- paste(A[,k],collapse=" //")
        }
      } else {
        print(i)
      }
    } 
tabS4$Transcript <- rownames(tabS4)
sum(tabA$Transcript%in%tabS4$Transcript)==dim(tabA)[1]
tabS4 <- merge(tabS4,tabA,all.x=TRUE)
tabS4$Transcript[is.na(tabS4$mapman.bin)]
temp <- matrix(unlist(strsplit(tabS4$Transcript,"_")),ncol=2,byrow=TRUE)
tabS4$Gene <- temp[,1]
tabS4 <- merge(tabS4,Cand,by.x="Gene",by.y="gene",all.x=TRUE,all.y=FALSE)
col.orders <- c("Gene","Transcript","name","average","sd" ,"FEarly_V","FEarly_T","FEarly_R","FLate_V","FLate_T","FLate_R","FVLate_V","FVLate_T","FVLate_R","MEarly_V","MEarly_T","MEarly_R","MLate_T","MLate_R","contrast","pcagroup","bin.main","mapman.bin","bin.name","kegg1","kegg2","kegg3")     
myorder <- order(tabS4$Gene)
tabS4 <- tabS4[myorder,col.orders]
write.table(tabS4,file="tabS4.csv",row.names=FALSE,sep="\t")
    
###Enrichment analyses
enrichment <- function(candidate.list,reference.list){
    cand.a <- table(candidate.list)
    ref <- table(reference.list)
    cand <- ref*0
    cand[names(cand.a)] <- cand.a
    thechisq.test <- chisq.test(cand,p=ref/sum(ref), simulate.p.value = TRUE, B=5000)  
    result <- list(pvalue=thechisq.test$p.value,residuals=thechisq.test$stdres)
  }  
  
# mapman
ref <- as.character(kegg$bin.main)
    
#by PCA axis    
tabcomp <- NULL  
  for (g in 1:4){  
    test1 <- as.character(kegg$bin.main)[as.character(kegg$Tr.name)%in%names(acpcol)[acpcol==g]]
    RDE1 <- enrichment(test1[!is.na(test1)],ref)
    tabcomp <- cbind(tabcomp,RDE1$residuals)
    print(RDE1$pvalue)
  }  
#for Selection category
test5 <- as.character(kegg$bin.main)[as.character(kegg$Tr.name)%in%ELgenes]
RDE5 <- enrichment(test5[!is.na(test5)],ref)
print(RDE5$pvalue)
tabcomp <- cbind(tabcomp,RDE5$residuals)

# mapman DEgenes with annotation
test6<- c(names(lineonly[lineonly==TRUE]),names(LinePlusYear[lineonly==TRUE]))
test7 <- as.character(kegg$bin.main)[as.character(kegg$Tr.name)%in%test6]
RDE7 <- enrichment(test7[!is.na(test7)],ref)
print(RDE7$pvalue)
tabcomp <- cbind(tabcomp,RDE7$residuals)
tab_mapman <- tabcomp
   
# kegg
ref <- as.character(kegg$kegg2)
   
#by PCA axis 
tabcomp <- NULL  
   for (g in 1:4){  
     test1 <- as.character(kegg$kegg2)[as.character(kegg$Tr.name)%in%names(acpcol)[acpcol==g]]
     RDE1 <- enrichment(test1[!is.na(test1)],ref)
     tabcomp <- cbind(tabcomp,RDE1$residuals)
     print(RDE1$pvalue)
   }  

#for Selection category
test5 <- as.character(kegg$kegg2)[as.character(kegg$Tr.name)%in%ELgenes]
RDE5 <- enrichment(test5[!is.na(test5)],ref)
print(RDE5$pvalue)
tabcomp <- cbind(tabcomp,RDE5$residuals)

### kegg DEgenes with annotation
test6<- c(names(lineonly[lineonly==TRUE]),names(LinePlusYear[lineonly==TRUE]))
test7 <- as.character(kegg$kegg2)[as.character(kegg$Tr.name)%in%test6]
RDE7 <- enrichment(test7[!is.na(test7)],ref)
print(RDE7$pvalue)
tabcomp <- cbind(tabcomp,RDE7$residuals)
tab_kegg <- tabcomp
write.table(tab_mapman, file = "MapMan_TableS6.csv", sep = "\t", na = "NA", dec = ".", row.names = TRUE, col.names = TRUE)
write.table(tab_kegg, file = "Kegg_TableS6.csv", sep = "\t", na = "NA", dec = ".", row.names = TRUE, col.names = TRUE)
   
###Heatmaps
myorder <- order(acpcor[,1],acpcor[,2])
hmcol <- c("blue","#74add1","#abd9e9","#ffffbf","#fdae61","#f46d43","red")
colnames(Zsignif) <- c("FE_R","FE_T","FE_V","FL_R","FL_T","FL_V","FVL_R","FVL_T","FVL_V","ME_R","ME_T","ME_V","ML_R","ML_T")
Zsignif <- Zsignif[,c("FE_V","FE_T","FE_R","FL_V","FL_T","FL_R","FVL_V","FVL_T","FVL_R","ME_V","ME_T","ME_R","ML_T","ML_R")]
qz <- quantile(as.matrix(Zsignif),probs=c(0.05,0.25,0.45,0.55,0.75,0.95))
Zg <- Zsignif[myorder,]
gg <- colnames(Zsignif)

K <- Zg*0
K[Zg<qz[1]] <- 1
K[Zg>=qz[1] & Zg < qz[2]] <- 2
K[Zg>=qz[2] & Zg < qz[3]] <- 3
K[Zg>=qz[3] & Zg < qz[4]] <- 4
K[Zg>=qz[4] & Zg < qz[5]] <- 5
K[Zg>=qz[5] & Zg < qz[6]] <- 6
K[Zg>=qz[6]] <- 7

thefile <- paste("Heatmaps.pdf")
pdf(thefile)
par(mfrow=c(1,1),mar=c(7,2,2,2))
image(x=1:dim(K)[2],y=1:dim(K)[1],z=as.matrix(t(K)),col=hmcol,xaxt="n",xlab="",ylab="")
  for (i in 1:length(gg)){
    mtext(side=1,at=i,line=1,gg[i],las=3)
  }
dev.off()		

### heatmaps by groups of genes
table(acpcol)
qz <- quantile(as.matrix(Zsignif),probs=c(0.05,0.25,0.45,0.55,0.75,0.95))
groups <- c("F252 vs MBS","F252 stages", "MBS stages & VL F252","Floral transition genes","GxS")

for (mygroup in 1:length(groups)){
genelist <- names(acpcol)[acpcol==mygroup]
Zg <- as.matrix(Zsignif)[genelist,]
myorder <- order(acpcor[genelist,mygroup])
Zg <- Zg[myorder,]
mygenelist <- genelist[myorder]
qz <- quantile(Zg,probs=c(0.05,0.25,0.45,0.55,0.75,0.95))
  
K <- Zg*0
K[Zg<qz[1]] <- 1
K[Zg>=qz[1] & Zg < qz[2]] <- 2
K[Zg>=qz[2] & Zg < qz[3]] <- 3
K[Zg>=qz[3] & Zg < qz[4]] <- 4
K[Zg>=qz[4] & Zg < qz[5]] <- 5
K[Zg>=qz[5] & Zg < qz[6]] <- 6
K[Zg>=qz[6]] <- 7
  
thefile <- paste("heatmap_group",mygroup,".pdf",sep="")
pdf(thefile)
par(mfrow=c(1,1),mar=c(7,2,2,2))
image(x=1:dim(K)[2],y=1:dim(K)[1],z=as.matrix(t(K)),col=hmcol,xaxt="n",xlab="",ylab="")
  for (i in 1:length(gg)){
  mtext(side=1,at=i,line=1,gg[i],las=3)
  }
  dev.off()		
}
